import warnings
import os
import sys
import pandas as pd
import pyarrow as pa
import pyarrow.parquet as pq
from sqlalchemy import create_engine, MetaData, Table, func, select
import sqlalchemy.types as sq
import sqlalchemy.dialects as sd
from sqlalchemy import exc as sa_exc
import numpy as np


USER_NAME = os.getenv('PIPE_USER', None)
USER_PWD = os.getenv('PIPE_PWD', None)
HOST_NAME = os.getenv('PIPE_HOST', None)
DB_NAME = os.getenv('PIPE_DB', 'nucleus')
TABLE_NAME = os.getenv('PIPE_TABLE', None)
SCHEMA_NAME = os.getenv('PIPE_SCHEMA', None)
APPNAME = 'csv2parquet'
DRIVER_NAME = os.getenv('PIPE_DRIVER', 'postgresql+psycopg2')
CONNECTION_STRING = '{}://{}:{}@{}/{}?application_name="{}"'.format(
    DRIVER_NAME, USER_NAME, USER_PWD, HOST_NAME, DB_NAME, APPNAME)
PARTITIONS = os.getenv('PIPE_PARTITION', 4)
TARGET_ROOT_LOCAL = os.getenv('PIPE_TARGET_LOCAL', '~/')
CHUNK_SIZE = os.getenv('PIPE_CHUNKSIZE', '100000')

PA_BOOL = pa.bool_()
PA_FLOAT32 = pa.float32()
PA_FLOAT64 = pa.float64()
PA_INT8 = pa.int8()
PA_INT16 = pa.int16()
PA_INT32 = pa.int32()
PA_INT64 = pa.int64()
PA_STRING = pa.string()
PA_TIMESTAMP = pa.timestamp('ns')
PA_BINARY = pa.binary()

def convert(table: Table): 
    fields = []
    for column in table.columns:
        paType = None
        if type(column.type) in (sq.Float,sq.Numeric,sq.NUMERIC):
            paType = PA_FLOAT64
        elif type(column.type) in (sq.Unicode,sq.Text,sq.String,sq.VARCHAR,sq.TEXT,sq.CLOB):
            paType = PA_STRING
        elif type(column.type) in (sq.BigInteger,sq.Integer,sq.SmallInteger, sq.BIGINT, sq.INTEGER):
            paType = PA_INT64
        elif type(column.type) in (sq.Binary,sq.BLOB):
            paType = PA_BINARY
        elif type(column.type) in (sq.BOOLEAN, sq.Boolean):
            paType = PA_STRING
        elif type(column.type) in (sq.TIMESTAMP, sq.DATETIME, sq.DateTime,sd.postgresql.base.TIMESTAMP):
            paType = PA_STRING
        elif type(column.type) in (sq.Date,sq.Time,sq.DATE,sq.TIME):
            paType = PA_STRING
        else:
            raise Exception("unknown sql datatype: {}".format(column.type) )
        fields.append(pa.field(column.name, paType))
    return pa.schema(fields)


def append_to_parquet_table(dataframe, filepath=None, writer=None, paSchema=None, dtypeSchema=None):
    """Method writes/append dataframes in parquet format.

    This method is used to write pandas DataFrame as pyarrow Table in parquet format. If the methods is invoked
    with writer, it appends dataframe to the already written pyarrow table.

    :param dataframe: pd.DataFrame to be written in parquet format.
    :param filepath: target file location for parquet file.
    :param writer: ParquetWriter object to write pyarrow tables in parquet format.
    :return: ParquetWriter object. This can be passed in the subsequenct method calls to append DataFrame
        in the pyarrow Table
    """
    warnings.filterwarnings("ignore", category=DeprecationWarning)
    for colname in dtypeSchema:
        if dataframe[colname].dtype != dtype_schema[colname]:
            if dtype_schema[colname] is not np.bool:
                dataframe[colname] = dataframe[colname].astype(dtype_schema[colname], copy=False)
        if dtype_schema[colname] == np.str:
            dataframe[colname].replace('nan','', inplace=True)

    table = pa.Table.from_pandas(dataframe, schema=paSchema)
    if writer is None:
        writer = pq.ParquetWriter(where=filepath, 
                                  schema=table.schema, 
                                  compression="snappy")
    writer.write_table(table=table)
    return writer


def convertDtypes(table: Table):
    dtypes = {}
    for column in table.columns:
        if type(column.type) in (sq.Float,sq.Numeric,sq.NUMERIC):
            pdType = np.float64
        elif type(column.type) in (sq.Unicode,sq.Text,sq.String,sq.VARCHAR,sq.TEXT,sq.CLOB):
            pdType = np.str
        elif type(column.type) in (sq.BigInteger,sq.Integer,sq.SmallInteger, sq.BIGINT, sq.INTEGER):
            pdType = "Int64"
        elif type(column.type) in (sq.Binary,sq.BLOB):
            pdType = np.binary
        elif type(column.type) in (sq.BOOLEAN, sq.Boolean):
            pdType = np.bool
        elif type(column.type) in (sq.TIMESTAMP, sq.DATETIME, sq.DateTime,sd.postgresql.base.TIMESTAMP):
            pdType = np.object
        elif type(column.type) in (sq.Date,sq.Time,sq.DATE,sq.TIME):
            pdType = np.object
        else:
            raise Exception("unknown sql datatype: {}".format(column.type) )
        dtypes[column.name]=pdType
    return dtypes


def tableMetadata(connection: str, schema_name: str, table_name: str):
    with warnings.catch_warnings():
        warnings.simplefilter("ignore", category=sa_exc.SAWarning)
        db_engine = create_engine(connection)
        metadata = MetaData()
        sqlTable = Table(table_name, metadata, schema=schema_name, autoload=True,
                        autoload_with=db_engine)
        rowQuery = select([func.count()]).select_from(sqlTable)
        rows = db_engine.scalar(rowQuery)
        return sqlTable, rows


schema_name = SCHEMA_NAME
table_name = TABLE_NAME
parquet_root = '{}/target/{}/{}/{}/'.format(
                                           TARGET_ROOT_LOCAL,
                                           DB_NAME,
                                           schema_name,
                                           table_name)
if not os.path.exists(parquet_root):
    os.makedirs(parquet_root)

sqlTable, rows = tableMetadata(CONNECTION_STRING, schema_name, table_name)
rows_per_partition = int(round(rows / 8))
dtype_schema = convertDtypes(sqlTable)
paSchema = convert(sqlTable)
chunksize = min([int(CHUNK_SIZE), rows_per_partition])
writer = None
partition_row_count = 0
partition_number = 1
parquet_file = '{}{}{}.parquet'.format(parquet_root, table_name, partition_number)
for chunk in pd.read_csv(sys.stdin,
                         sep=',',
                         chunksize=chunksize,
                         iterator=True,
                         low_memory=False,
                         encoding='utf-8'):
    writer = append_to_parquet_table(chunk, parquet_file, writer, paSchema, dtype_schema);
    partition_row_count += len(chunk)
    if partition_row_count >= rows_per_partition:
        partition_row_count = 0
        partition_number += 1
        writer = None
        parquet_file = '{}/{}{}.parquet'.format(parquet_root, table_name, partition_number)



import warnings
from dataclasses import dataclass
from typing import Set, Optional
from sqlalchemy import create_engine, Table, Column, MetaData
from sqlalchemy.engine.url import make_url
from sqlalchemy import exc as sa_exc
from concurrent.futures import ThreadPoolExecutor
import os

@dataclass(unsafe_hash=True, eq=True)
class Interface:
    name: str
    schema_name: str
    table_name: Optional[str]
    unique_index: str
    description: str
    table: Optional[Table] = None
    full: bool = False


@dataclass(unsafe_hash=True, eq=True)
class Source:
    name: str
    description: str
    connection_url: str
    schema_name: Optional[str]
    interfaces: Optional[Set[Interface]] = None


@dataclass
class DBMSMeta:

    @staticmethod
    def read(source_name: str, table_name: str, 
             schema_name: str, 
             index_col: str, 
             connection_url: str):
        with warnings.catch_warnings():
            warnings.simplefilter("ignore", category=sa_exc.SAWarning)
            url = make_url(connection_url)
            url.username=os.environ['{}_USER'.format(source_name).upper()]
            url.password=os.environ['{}_PASS'.format(source_name).upper()]
            engine = create_engine(url)
            metadata = MetaData()
            table = Table(table_name, metadata, schema=schema_name, autoload=True, autoload_with=engine)
            for col in table.columns:
                if type(col.type).__name__ == 'TIMESTAMP' and connection_url.startswith('mssql'):
                    col.doc = 'NVARCHAR'
                else:
                    col.doc = type(col.type).__name__
            if index_col:
                ixcols = index_col.split(',')
                for col in ixcols:
                    if table.columns.has_key(col):
                        table.primary_key.columns[col] = table.columns[col]
                    else:
                        table.primary_key.columns[col] = table.columns[col.upper()]
            table.info = { "database": engine.url.database}
            return table





import json
import sys
from dataclasses import dataclass, field
from typing import List
from io import StringIO


@dataclass
class Column:
    column_name: str
    source_name: str = None
    data_type: str = 'string'
    is_key: bool = False
    is_hidden: bool = False

    @staticmethod
    def md_header(self) -> List[str]:
        return ['| Column | Source | Type | Key | Hidden |\n', 
                '| ------ | ------ | --- | --- | --- |\n']

    def md_rows(self) -> List[str]:
        return [ "|{0}|{1}|{2}|{3}|{4}|\n".format(self.column_name, self.source_name, self.data_type, self.is_key, self.is_hidden) ]


@dataclass
class Table:
    table_name: str
    columns: List[Column] = field(default_factory=list)
    sources: List[str] = field(default_factory=list)

    def md_header(self) -> List[str]:
        return [ "### Table: {0}\n".format(self.table_name) ]

    def md_column_table(self) -> List[str]:
        lines = Column.md_header(self)
        for col in self.columns:
            lines = lines + col.md_rows()
        return lines

    def md_partitions(self) -> List[str]:
        lines = [ "- Partition source: \n", "```\n" ]
        for src in self.sources:
            for line in src.get('expression', '').splitlines():
                lines.append("{0}\n".format(line)) 
        return lines + [ "\n```\n" ]


@dataclass
class Query:
    name: str
    sql: str = None
    database: str = None
    query_group: str = None
    expression: List[str] = field(default_factory=list)
    
    def md_header(self) -> List[str]:
        return [
            "### Query: {0}\n".format(self.name),
            "\n",
            "- Database: {0}\n".format(self.database),
            "- Query Group: {0}\n".format(self.query_group),
            "- Sql: \n"
        ]

    def md_sql(self) -> List[str]:
        return [
            "```\n",
            "{0}\n".format(self.sql),
            "```\n"
        ]

    def md_expression(self) -> List[str]:
        list = [ "- Expressions: \n", "```\n" ]
        for exp in self.expression:
            list = list + [ "{0}\n".format(exp) ]
        return list + [ "```\n" ]


@dataclass
class Report:
    report_name: str
    queries: List[Query] = None
    tables: List[Table] = None

    def render_md(self) -> StringIO:
        output = StringIO()
        output.writelines( [ "# {0}\n".format(self.report_name), " \n" ])
        output.writelines( [ "## Expressions\n" ])
        if self.queries:
            for query in self.queries:
                output.writelines(query.md_header())
                output.writelines(query.md_sql())
                output.writelines(query.md_expression())
        if self.tables:
            output.writelines( [ "## Tables\n" ])
            for table in self.tables:
                output.writelines(table.md_header())
                output.writelines(table.md_column_table())
                output.writelines("\n")
                output.writelines(table.md_partitions())
        return output


def parse_query(json_model) -> List[Query]:
    if not 'expressions' in json_model:
        return None
    queries = []
    for exp in json_model['expressions']:
        query = Query(exp['name'])
        queries.append(query)
        expStr = exp['expression']
        idb = int(expStr.find('Database("'))
        edb = int(expStr.find('\"]),'))
        isql = int(expStr.find('Query="'))
        query.database = expStr[idb+9:isql-2]
        sql = expStr[isql+7:edb].replace('\n', '')
        sql = sql.replace('#(lf)', '\n')
        sql = sql.replace('#(tab)', '\t')
        query.sql = sql
        expressions = expStr[edb+5:len(expStr)]
        for line in expressions.splitlines():
            if line.find('#') > 0:
                query.expression.append(line)
        if 'queryGroup' in exp:
            query.query_group = exp['queryGroup']
    return queries


def parse_table(json_model) -> List[Table]:
    if not 'tables' in json_model:
        return
    tables = []
    for tab in json_model['tables']:
        table = Table(tab['name'])
        tables.append(table)
        for col in tab['columns']:
            column = Column(col['name'])
            column.data_type = col.get('dataType', 'string')
            column.is_key = col.get('isKey', False)
            column.is_hidden = col.get('isHidden', False)
            column.source_name = col.get('sourceColumn', None)
            table.columns.append(column)
        for part in tab['partitions']:
            table.sources.append(part['source'])
    return tables


def main():
    data = json.load(sys.stdin)
    model = data['model']
    report = Report('reportname')
    report.queries = parse_query(model)
    report.tables = parse_table(model)
    print(report.render_md().getvalue())


if __name__ == '__main__':
    sys.exit(main())



import json
import yaml
from dataclasses import Field, dataclass, field
from typing import Dict, List, Set, Optional, Text
from enum import Enum
from pydantic import BaseModel, validator, Field
from pydantic.errors import PathError
from pathlib import Path
import glob
import subprocess as sp
import os
from os.path import dirname
import logging
import sys
import concurrent.futures
from meta import Source, Interface
from sqlalchemy import Table

project_exceptions = ['dbt_utils', 'dbt_expectations']

logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)

LOG = logging.getLogger('dbt_artifact')


class DbtResourceType(str, Enum):
    model = 'model'
    analysis = 'analysis'
    test = 'test'
    operation = 'operation'
    seed = 'seed'
    source = 'source'


class DbtMaterializationType(str, Enum):
    table = 'table'
    view = 'view'
    incremental = 'incremental'
    ephemeral = 'ephemeral'
    seed = 'seed'
    test = 'test'


class NodeDeps(BaseModel):
    nodes: List[str]


class NodeConfig(BaseModel):
    materialized: Optional[DbtMaterializationType]


class Node(BaseModel):
    unique_id: str
    name: str
    alias: Optional[str]
    database: str
    schema_name: str = Field(alias='schema')
    package_name: str
    original_file_path: str
    path: Path
    root_path: Path
    resource_type: DbtResourceType
    description: str
    fqn: Optional[List]
    tags: Optional[List]
    depends_on: Optional[NodeDeps]
    config: NodeConfig
    meta: Optional[Dict]
    source_name: Optional[str]
    source_description: Optional[str]
    source_meta: Optional[Dict]
    

class Manifest(BaseModel):
    nodes: Dict["str", Node]
    sources: Dict["str", Node]

    @validator('nodes', 'sources')
    def filter(cls, val):
        return {k: v for k, v in val.items() if v.resource_type.value in ('model', 'seed', 'source')}


class DbtProjectType(str, Enum):
    psa = 'psa'
    hardrule = 'hardrule'
    datavault = 'datavault'
    softrule = 'softrule'
    dimensional = 'dimensional'


@dataclass(unsafe_hash=True, eq=True)
class DbtProject():
    project_path: str = ''
    project_cfg: str = None
    project_type: DbtProjectType = None
    project_name: str = ''
    source_paths: List[str] = field(default_factory=list)
    data_paths: List[str] = field(default_factory=list)
    test_paths: List[str] = field(default_factory=list)
    analysis_paths: List[str] = field(default_factory=list)
    macro_paths: List[str] = field(default_factory=list)
    snapshot_paths: List[str] = field(default_factory=list)
    target_path: str = 'target'
    log_path: str = 'logs'
    modules_path: str = 'dbt_modules'
    initialized: bool = False
    manifest: Manifest = None


    @staticmethod
    def create_deck_hardrule(project_path: str, source_filter):
        dbt_files = glob.glob(project_path + "/00_src/**/dbt_project.yml", recursive=False)
        hardrules = []
        for dbt_file in dbt_files:
            with open(dbt_file, 'r') as f:
                py = yaml.safe_load(f)
                dbtproj = DbtProject(
                    project_path=dirname(f.name),
                    project_cfg=py
                )
                if dbtproj.project_type == DbtProjectType.hardrule and dbtproj.project_name in source_filter:
                    hardrules.append(dbtproj)
        return hardrules


    @staticmethod
    def create_psa(project_path: str):
        dbt_file = glob.glob(project_path + "/**/psa/dbt_project.yml", recursive=True)
        if len(dbt_file) == 1:
            with open(dbt_file[0], 'r') as f:
                py = yaml.safe_load(f)
                dbtproj = DbtProject(
                    project_path=dirname(f.name),
                    project_cfg=py
                )
                if dbtproj.project_type == DbtProjectType.psa:
                    return dbtproj


    @staticmethod
    def create_deck(project_path: str, prj_except: List):
        dbt_files = glob.glob(project_path + "/**/dbt_project.yml", recursive=True)
        dbts = {}
        hardrules = []
        datavaults = []
        psas = []
        softrules = []
        dimensionals = []
        dbts[DbtProjectType.psa]=psas
        dbts[DbtProjectType.hardrule]=hardrules
        dbts[DbtProjectType.datavault]=datavaults
        dbts[DbtProjectType.softrule]=softrules
        dbts[DbtProjectType.dimensional]=dimensionals
        for dbt_file in dbt_files:
            if not dirname(dirname(dbt_file)) in prj_except:
                with open(dbt_file, 'r') as f:
                    py = yaml.safe_load(f)
                    dbtproj = DbtProject(
                        project_path=dirname(f.name),
                        project_cfg=py
                    )
                    if dbtproj.project_type == DbtProjectType.hardrule:
                        hardrules.append(dbtproj)
                    elif dbtproj.project_type == DbtProjectType.psa:
                        psas.append(dbtproj)
                    elif dbtproj.project_type == DbtProjectType.datavault:
                        datavaults.append(dbtproj)
                    elif dbtproj.project_type == DbtProjectType.softrule:
                        softrules.append(dbtproj)
                    elif dbtproj.project_type == DbtProjectType.dimensional:
                        dimensionals.append(dbtproj)
        return dbts


    def __post_init__(self):
        if 'models' in self.project_cfg:
            #if not '+tags' in self.project_cfg['models']:
            #    raise RuntimeError('In {0} type (psa,hardrule,datavault,softrule,dimensional) must be set as +tags entry'.format(self.project_path))

            if '+tags' in self.project_cfg['models']:
                if DbtProjectType.hardrule in self.project_cfg['models']['+tags']:
                    self.project_type = DbtProjectType.hardrule
                elif DbtProjectType.psa in self.project_cfg['models']['+tags']:
                    self.project_type = DbtProjectType.psa
                elif DbtProjectType.datavault in self.project_cfg['models']['+tags']:
                    self.project_type = DbtProjectType.datavault
                elif DbtProjectType.softrule in self.project_cfg['models']['+tags']:
                    self.project_type = DbtProjectType.softrule
                elif DbtProjectType.dimensional in self.project_cfg['models']['+tags']:
                    self.project_type = DbtProjectType.dimensional
                else:
                    raise RuntimeError('In {0} type (psa,hardrule,datavault,softrule,dimensional) must be set as +tags entry'.format(self.project_path))

        self.project_name = self.project_cfg['name']

        if 'source-paths' in self.project_cfg:
            self.source_paths = self.project_cfg['source-paths']
        else:
            self.source_paths = ['models']
        if 'data-paths' in self.project_cfg:
            self.data_paths = self.project_cfg['data-paths']
        else:
            self.data_paths = ['data']
        if 'test-paths' in self.project_cfg:
            self.test_paths = self.project_cfg['test-paths']
        else:
            self.test_paths = ['test']
        if 'analysis-paths' in self.project_cfg:
            self.analysis_paths = self.project_cfg['analysis-paths']
        else:
            self.analysis_paths = ['analysis']
        if 'macro-paths' in self.project_cfg:
            self.macro_paths = self.project_cfg['macro-paths']
        else:
            self.macro_paths = ['macros']
        if 'snapsho-paths' in self.project_cfg:
            self.snapshot_paths = self.project_cfg['snapshot-paths']
        else:
            self.snapshot_paths = ['snapshots']

        if 'modules_path' in self.project_cfg:
            self.modules_path = self.project_cfg['modules_path']
        if 'target-path' in self.project_cfg:
            self.target_path = self.project_cfg['target-path']
        if 'log-path' in self.project_cfg:
            self.log_path = self.project_cfg


    def sources(self):
        srcs = {}
        if self.initialized:
            for source in self.manifest.sources:
                src_node = self.manifest.sources[source]
                if 'dburl' in src_node.source_meta:
                    src = None
                    if not src_node.source_name in srcs.keys():
                        src = Source(schema_name=src_node.schema_name,
                            name=src_node.source_name,
                            description=src_node.source_description,
                            connection_url=src_node.source_meta['dburl'])
                        srcs[src.name] = src
                    else:
                        src = srcs[src_node.source_name]
                    if src_node.meta:
                        interface = Interface(name=src_node.name,
                            schema_name=src_node.meta.get('source_schema', ''),
                            table_name=src_node.meta.get('source_name', src_node.name),
                            unique_index=src_node.meta.get('unique_index', ''),
                            description=src_node.description)
                        if src_node.meta.get('full'):
                            interface.full = True
                        if not src.interfaces:
                            src.interfaces = set()
                        src.interfaces.add(interface)
        return srcs


    def initialize(self):
        cmd = ['dbt', 'deps']
        cmp_rc, cmp_out = self.exec_dbt(cmd)
        if cmp_rc != 0:
            return cmp_rc, cmp_out

        cmd = ['dbt', 'compile']
        cmp_rc, cmp_out = self.exec_dbt(cmd)
        if cmp_rc == 0:
            mj = self.project_path + '/' + self.target_path + '/' + 'manifest.json'
            with open(mj, 'r') as fh:
                data = json.load(fh)
            self.manifest = Manifest(**data)
            self.initialized = True
        return cmp_rc, cmp_out


    def run(self):
        if not self.initialized:
            self.initialize()
        cmd = ['dbt', 'run']
        return self.exec_dbt(cmd)

    def test(self):
        if not self.initialized:
            self.run()
        cmd = ['dbt', 'run']
        return self.exec_dbt(cmd)

    def docs(self):
        if not self.initialized:
            self.initialize()
        cmd = ['dbt', 'docs', 'generate']
        return self.exec_dbt(cmd)

    def exec_dbt(self, cmd):
        cmd = cmd + ['--project-dir', self.project_path,
                     '--profiles-dir', self.project_path]
        LOG.debug(cmd)
        with sp.Popen(cmd, stdout=sp.PIPE, stderr=sp.STDOUT) as process:
            try:
                outs, errs = process.communicate(timeout=15)
                returncode = process.wait()
                if returncode == 0:
                    LOG.info('DbtProjet {0} {1} succeeded.'.format(
                        cmd[1], self.project_name))
                else:
                    LOG.error('DbtProjet {0} {1} failed.'.format(
                        cmd[1], self.project_name))
                    LOG.error(outs)

                return returncode, outs
            except sp.TimeoutExpired:
                process.kill()
                outs, errs = process.communicate()
                return -1, outs


def initialize_dbt(dbtProject):
    return dbtProject.initialize()




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dwhgen']

package_data = \
{'': ['*'],
 'dwhgen': ['template/common/template/exasol/initial/*',
            'template/common/template/exasol/interface/*',
            'template/common/templates/dbt/*',
            'template/common/templates/psa/*',
            'template/common/templates/source_schema/*',
            'template/common/templates_dbt/src/*',
            'template/common/templates_dbt/stg/*',
            'template/exasol/templates/dbt/*',
            'template/exasol/templates_dbt/stg/*']}

install_requires = \
['HiYaPyCo>=0.4.16,<0.5.0',
 'SQLAlchemy==1.3.23',
 'dbt-exasol>=1.0.3,<2.0.0',
 'dbt>=0.21.1,<0.22.0',
 'ibm-db-sa>=0.3.7,<0.4.0',
 'markupsafe==2.0.1',
 'pydantic>=1.8.2,<2.0.0',
 'pyexasol>=0.21.1,<0.22.0',
 'pymssql>=2.2.2,<3.0.0',
 'pysqlite3>=0.4.6,<0.5.0',
 'python-dotenv>=0.19.0,<0.20.0',
 'sqlalchemy-exasol>=2.2.0,<3.0.0',
 'sqlalchemy-pysqlite3>=0.0.4,<0.0.5',
 'sqlfluff>=0.8.1,<0.9.0',
 'yamlreader>=3.0.4,<4.0.0']

entry_points = \
{'console_scripts': ['dwhgen = dwhgen.main:main']}

setup_kwargs = {
    'name': 'dwhgen',
    'version': '0.1.2',
    'description': 'generating schema YAML and models for source system interfaces and PSA historization',
    'long_description': None,
    'author': 'Torsten Glunde',
    'author_email': 'torsten@glunde.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import argparse
import logging
import logging.config
import os
import sys
import tempfile
import time
from pathlib import Path

from dotenv import find_dotenv, load_dotenv
from jinja2 import Environment, FileSystemLoader, select_autoescape

from dbtparse import DbtProject, DbtProjectType
from meta import DBMSMeta

APPNAME="dbt-dwhgen"
this_dir, this_filename = os.path.split(__file__)
TEMPLATE_PATH = '{}{}'.format(this_dir, '/template/exasol/templates')
TEMPLATE_PATH_DBT = '{}{}'.format(this_dir, '/template/exasol/templates_dbt')
TARGET_PATH='./target'
LOG_LEVEL=logging.INFO
LOG = logging.getLogger(APPNAME)


def main():
    load_dotenv(find_dotenv())
    parser = argparse.ArgumentParser()
    parser.add_argument('-s', '--sources', required=True)
    parser.add_argument('-p' '--project', required=True)
    parser.add_argument('-d', '--database', default='exasol')

    args = parser.parse_args()
    TEMPLATE_PATH = '{}/template/{}/templates'.format(this_dir, args.database)
    TEMPLATE_PATH_DBT = '{}/template/{}/templates_dbt'.format(this_dir, args.database)
    TARGET_PATH='{}/dwhgen'.format(tempfile.gettempdir())
    os.makedirs(TARGET_PATH)
    LOG.info('Temporary target path: {}'.format(TARGET_PATH))
    source_list = args.sources.split(',')
    dbt_deck = DbtProject.create_deck_hardrule(args.p__project, source_list  )

    tables = []
    sources = {}
    for dbt in dbt_deck:
        rc, output = dbt.initialize()
        if rc != 0:
            LOG.error(output)
            return -1
            
        localsrc = dbt.sources()
        
        for source in localsrc.values():
            sources[source.name] = source
            for interface in source.interfaces:                
                table = DBMSMeta.read(source.name, interface.table_name,
                    interface.schema_name,
                    interface.unique_index,
                    source.connection_url)
                interface.table = table
                tables.append(table)

    start = time.time()

    LOG.info("Starting templates:")
    for table in tables:
        env = Environment(
            loader=FileSystemLoader(TEMPLATE_PATH),
            autoescape=select_autoescape(['sql','yml'])
        )
        for k,v in os.environ.items():
            env.globals[k]=v

        result = list(Path(TEMPLATE_PATH).glob('**/*.sql'))
        result.extend(list(Path(TEMPLATE_PATH).glob('**/*.yml')))
        for entry in result:
            template = env.get_template(str(entry.relative_to(TEMPLATE_PATH).as_posix())).render(tables=tables,sources=sources)
            target_dir = '{}/{}/'.format(TARGET_PATH, entry.parent.name)
            if not Path(target_dir).exists():
                os.makedirs(target_dir)
            with open('{}/{}'.format(target_dir, entry.name), 'w') as f:
                f.write(template)

        env = Environment(
            loader=FileSystemLoader(TEMPLATE_PATH_DBT),
            autoescape=select_autoescape(['sql','yml'])
        )
        for k,v in os.environ.items():
            env.globals[k]=v
        for source in sources.values():
            for interface in source.interfaces:
                template = env.get_template('stg/stg.sql').render(source=source,interface=interface)
                target_dir = '{}/dbt/stg/'.format(TARGET_PATH)
                if not Path(target_dir).exists():
                    os.makedirs(target_dir)
                with open('{}/stg_{}_{}_{}.sql'.format(target_dir, source.name, interface.table.schema, interface.table.name), 'w') as f:
                    f.write(template)

                #template = env.get_template('src/psa_src.sql').render(source=source,interface=interface)
                target_dir = '{}/dbt/'.format(TARGET_PATH)
                if not Path(target_dir).exists():
                    os.makedirs(target_dir)
                with open('{}/{}_{}_{}.sql'.format(target_dir, source.name, interface.table.schema, interface.table.name), 'w') as f:
                    f.write(template)
                    
    end = time.time()
    LOG.info('Execution finished in {}.'.format((end-start)))


if __name__ == '__main__':
    sys.exit(main())



from typing import List, Set
from dataclasses import dataclass
from dataclasses_json import dataclass_json
import os


@dataclass_json
@dataclass
class JdbcDataType:
    character_maximum_length: int
    data_type_name: str
    jdbc_data_type_id: int
    numeric_precision: int
    numeric_scale: int


@dataclass_json
@dataclass
class Column:
    column_id: str
    column_order: int
    column_comment: str
    column_name: str
    complete_data_type: str
    jdbc_data_type: JdbcDataType


@dataclass_json
@dataclass
class Hub:
    hub_id: str
    hub_comment: str
    hub_id_of_alias_parent: str
    hub_name_of_alias_parent: str
    hub_subject_area_name: str


@dataclass_json
@dataclass
class HubLoad:
    business_key_prefix: str
    business_key_short: str
    columns: Set[Column]
    datavault_category_id: str
    hub_id: str
    hub_load_display_name: str
    hub_load_id: str
    keys_are_prehashed: bool
    keys_are_unique: bool
    staging_table_id: str
    system_id: str
    system_name: str


@dataclass_json
@dataclass
class Satellite:
    column_ids: List
    functional_suffix_id: str
    functional_suffix_name: str
    hub_id: str
    satellite_comment: str
    satellite_id: str
    satellite_is_prototype: bool
    satellite_subject_area_name: str
    staging_resource_id: str
    staging_table_id: str
    system_id: str


@dataclass_json
@dataclass
class LinkLoad:
    link_id: str
    link_load_display_name: str
    link_load_id: str
    staging_table_id: str
    system_id: str


@dataclass_json
@dataclass
class Link:
    hub_a_id: str
    hub_b_id: str
    link_comment: str
    link_id: str
    link_subject_area_name: str
    link_suffix_id: str
    link_suffix_name: str
    link_type: str


@dataclass_json
@dataclass
class SourceSystem:
    deployable: bool
    hidden_in_system_deployment: bool
    source_type_id: str
    source_type_name: str
    source_type_parameters: List
    source_type_url: str
    system_color: str
    system_comment: str
    system_id: str
    system_name: str


@dataclass_json
@dataclass
class StagingTable:
    columns: List[Column]
    loading_batch_size: int
    source_schema_name: str
    source_schema_or_system_id: str
    source_table_id: str
    source_table_name: str
    source_table_staging_id: str
    source_table_type_id: str
    staging_table_comment: str
    staging_table_id: str
    staging_table_name: str
    system_id: str
    where_clause_delta_part_template: str
    where_clause_general_part: str


path = r"/home/torsten/project/dwh_2020/datavault/dvb/datavaultbuilder_objects/staging/staging_tables/"
directories = os.listdir( path )

for file in os.scandir(path):  
    if file.is_file():
        fs = open("{}/{}".format(path,file.name),'r')
        stg = StagingTable.from_json(fs.read())
        print(stg)

